package com.fr.plugin.db.redis.help.client;

import com.eclipsesource.v8.V8;
import com.eclipsesource.v8.V8Array;
import com.eclipsesource.v8.V8Object;
import com.fanruan.api.log.LogKit;
import com.fr.base.TableData;
import com.fr.plugin.db.redis.core.accessor.EmbedRedis;
import com.fr.plugin.db.redis.core.emb.Redis;
import com.fr.plugin.db.redis.help.Console;
import com.fr.plugin.db.redis.help.ScriptBridge;
import com.fr.plugin.db.redis.util.RedisUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @author richie
 * @version 10.0
 * Created by richie on 2019-03-18
 */
public class RedisV8Client extends BaseRedisClient<V8Array> {

    private V8 v8;

    public RedisV8Client(EmbedRedis jedis) {
        super(jedis);
        this.v8 = V8.createV8Runtime();
    }

    @Override
    @ScriptBridge
    public String mget(V8Array keys) {
        long start = System.currentTimeMillis();
        int len = keys.length();
        String[] parameters = new String[len];
        for (int i = 0; i < len; i++) {
            parameters[i] = keys.getString(i);
        }
        List<String> list = jedis.mget(parameters);
        LogKit.info("Fetch data from redis spend time {} ms.", System.currentTimeMillis() - start);
        keys.release();
        return object2JSONString(list);
    }

    @Override
    public SimpleDataModel build(Redis redis, String script, int limit) {
        RedisV8Client client = new RedisV8Client(redis.getClient());
        V8Object v8Console = initConsole(v8);
        V8Object v8Require = initRequire(v8);
        V8Object v8Client = initRedis(v8, client);

        v8.executeVoidScript(
                "function unique(array) {return Array.from(new Set(array));};" +
                        "function merge(table, column) {return {content:table, column:column}};");


        V8Object returnObj = v8.executeObjectScript(String.format("(function(){%s})();", script));

        V8Array v8Column = returnObj.getArray("column");
        V8Array v8Table = returnObj.getArray("content");
        int columnCount = v8Column.length();
        String[] columnNames = new String[columnCount];
        for (int i = 0; i < columnCount; i++) {
            columnNames[i] = String.valueOf(v8Column.get(i));
        }
        int returnRowCount = v8Table.length();
        int realCount = limit == TableData.RESULT_ALL ? returnRowCount : Math.min(returnRowCount, limit);
        List<List<Object>> data = new ArrayList<List<Object>>();
        for (int i = 0; i < realCount; i++) {
            V8Array v8Row = v8Table.getArray(i);
            List<Object> row = new ArrayList<Object>();
            for (int j = 0, col = v8Row.length(); j < col; j++) {
                row.add(v8Row.get(j));
            }
            v8Row.release();
            data.add(row);
        }
        redis.close();
        v8Table.release();
        v8Column.release();
        returnObj.release();
        v8Client.release();
        v8Require.release();
        v8Console.release();

        v8.release(true);
        return new SimpleDataModel(columnNames, data, realCount);
    }

    private V8Object initConsole(V8 v8) {
        V8Object v8Console = new V8Object(v8);
        v8.add("console", v8Console);
        Console console = new Console();
        RedisUtils.registerJavaMethods(v8Console, console);
        return v8Console;
    }

    private V8Object initRequire(V8 v8) {
        V8Object v8Files = new V8Object(v8);
        v8.add("Files", v8Files);
        V8Files console = new V8Files(v8);
        RedisUtils.registerJavaMethods(v8Files, console);
        return v8Files;
    }

    private V8Object initRedis(final V8 v8, final RedisV8Client client) {
        V8Object v8Client = new V8Object(v8);
        v8.add("redis", v8Client);
        RedisUtils.registerJavaMethods(v8Client, client);
        return v8Client;
    }
}
